/*
 * Copyright (c) Huawei Technologies Co., Ltd. 2020-2024. All rights reserved.
 */

#ifndef DPC_IMPL_TYPE_DECISION4LAT_H
#define DPC_IMPL_TYPE_DECISION4LAT_H

#include <cfloat>
#include <cmath>
#include "impl_type_uint8.h"
#include "impl_type_float.h"
#include "impl_type_uint32.h"
#include "impl_type_uint64.h"

namespace dpc {
struct Decision4Lat {
    ::UInt8 ba_trp_lat_sts{ 0U };
    ::UInt8 ba_lat_actv_func_sts{ 0U };
    ::Float ba_ref_line_c0{ 0.0F };
    ::Float ba_ref_line_c1{ 0.0F };
    ::Float ba_ref_line_c2{ 0.0F };
    ::Float ba_ref_line_c3{ 0.0F };
    ::Float ba_ref_line_rng_sts{ 0.0F };
    ::Float ba_ref_line_rng_ed{ 0.0F };
    ::Float ba_ego_lane_width{ 0.0F };
    ::UInt8 ba_drive_md{ 0U };
    ::UInt8 ba_lat_fd_in_md{ 0U };
    ::UInt8 ba_lat_fd_out_md{ 0U };
    ::UInt8 ba_hd_feel_md{ 0U };
    ::UInt8 ba_opt_action{ 0U };
    ::UInt32 ba_opt_corridor_id{ 0U };
    ::UInt32 ba_rc4trp_lat{ 0U };
    ::UInt64 ba_fusion_time_stamp{ 0U };

    static bool IsPlane()
    {
        return true;
    }


    using IsEnumerableTag = void;
    template<typename F>
    void enumerate(F& fun)
    {
        fun(ba_trp_lat_sts);
        fun(ba_lat_actv_func_sts);
        fun(ba_ref_line_c0);
        fun(ba_ref_line_c1);
        fun(ba_ref_line_c2);
        fun(ba_ref_line_c3);
        fun(ba_ref_line_rng_sts);
        fun(ba_ref_line_rng_ed);
        fun(ba_ego_lane_width);
        fun(ba_drive_md);
        fun(ba_lat_fd_in_md);
        fun(ba_lat_fd_out_md);
        fun(ba_hd_feel_md);
        fun(ba_opt_action);
        fun(ba_opt_corridor_id);
        fun(ba_rc4trp_lat);
        fun(ba_fusion_time_stamp);
    }

    template<typename F>
    void enumerate(F& fun) const
    {
        fun(ba_trp_lat_sts);
        fun(ba_lat_actv_func_sts);
        fun(ba_ref_line_c0);
        fun(ba_ref_line_c1);
        fun(ba_ref_line_c2);
        fun(ba_ref_line_c3);
        fun(ba_ref_line_rng_sts);
        fun(ba_ref_line_rng_ed);
        fun(ba_ego_lane_width);
        fun(ba_drive_md);
        fun(ba_lat_fd_in_md);
        fun(ba_lat_fd_out_md);
        fun(ba_hd_feel_md);
        fun(ba_opt_action);
        fun(ba_opt_corridor_id);
        fun(ba_rc4trp_lat);
        fun(ba_fusion_time_stamp);
    }

    template<typename F>
    void enumerate_internal(F& fun)
    {
        fun("ba_trp_lat_sts", ba_trp_lat_sts);
        fun("ba_lat_actv_func_sts", ba_lat_actv_func_sts);
        fun("ba_ref_line_c0", ba_ref_line_c0);
        fun("ba_ref_line_c1", ba_ref_line_c1);
        fun("ba_ref_line_c2", ba_ref_line_c2);
        fun("ba_ref_line_c3", ba_ref_line_c3);
        fun("ba_ref_line_rng_sts", ba_ref_line_rng_sts);
        fun("ba_ref_line_rng_ed", ba_ref_line_rng_ed);
        fun("ba_ego_lane_width", ba_ego_lane_width);
        fun("ba_drive_md", ba_drive_md);
        fun("ba_lat_fd_in_md", ba_lat_fd_in_md);
        fun("ba_lat_fd_out_md", ba_lat_fd_out_md);
        fun("ba_hd_feel_md", ba_hd_feel_md);
        fun("ba_opt_action", ba_opt_action);
        fun("ba_opt_corridor_id", ba_opt_corridor_id);
        fun("ba_rc4trp_lat", ba_rc4trp_lat);
        fun("ba_fusion_time_stamp", ba_fusion_time_stamp);
    }

    template<typename F>
    void enumerate_internal(F& fun) const
    {
        fun("ba_trp_lat_sts", ba_trp_lat_sts);
        fun("ba_lat_actv_func_sts", ba_lat_actv_func_sts);
        fun("ba_ref_line_c0", ba_ref_line_c0);
        fun("ba_ref_line_c1", ba_ref_line_c1);
        fun("ba_ref_line_c2", ba_ref_line_c2);
        fun("ba_ref_line_c3", ba_ref_line_c3);
        fun("ba_ref_line_rng_sts", ba_ref_line_rng_sts);
        fun("ba_ref_line_rng_ed", ba_ref_line_rng_ed);
        fun("ba_ego_lane_width", ba_ego_lane_width);
        fun("ba_drive_md", ba_drive_md);
        fun("ba_lat_fd_in_md", ba_lat_fd_in_md);
        fun("ba_lat_fd_out_md", ba_lat_fd_out_md);
        fun("ba_hd_feel_md", ba_hd_feel_md);
        fun("ba_opt_action", ba_opt_action);
        fun("ba_opt_corridor_id", ba_opt_corridor_id);
        fun("ba_rc4trp_lat", ba_rc4trp_lat);
        fun("ba_fusion_time_stamp", ba_fusion_time_stamp);
    }

    friend bool operator==(const ::dpc::Decision4Lat& lhs, const ::dpc::Decision4Lat& rhs) noexcept
    {
        return (lhs.ba_trp_lat_sts == rhs.ba_trp_lat_sts) && (lhs.ba_lat_actv_func_sts == rhs.ba_lat_actv_func_sts) && (fabs(static_cast<double>(lhs.ba_ref_line_c0 - rhs.ba_ref_line_c0)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.ba_ref_line_c1 - rhs.ba_ref_line_c1)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.ba_ref_line_c2 - rhs.ba_ref_line_c2)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.ba_ref_line_c3 - rhs.ba_ref_line_c3)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.ba_ref_line_rng_sts - rhs.ba_ref_line_rng_sts)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.ba_ref_line_rng_ed - rhs.ba_ref_line_rng_ed)) < DBL_EPSILON) && (fabs(static_cast<double>(lhs.ba_ego_lane_width - rhs.ba_ego_lane_width)) < DBL_EPSILON) && (lhs.ba_drive_md == rhs.ba_drive_md) && (lhs.ba_lat_fd_in_md == rhs.ba_lat_fd_in_md) && (lhs.ba_lat_fd_out_md == rhs.ba_lat_fd_out_md) && (lhs.ba_hd_feel_md == rhs.ba_hd_feel_md) && (lhs.ba_opt_action == rhs.ba_opt_action) && (lhs.ba_opt_corridor_id == rhs.ba_opt_corridor_id) && (lhs.ba_rc4trp_lat == rhs.ba_rc4trp_lat) && (lhs.ba_fusion_time_stamp == rhs.ba_fusion_time_stamp);
    }
};
} // namespace dpc


#endif // DPC_IMPL_TYPE_DECISION4LAT_H
